-- **************************************************************************
--  This is Analog ADF4380-8 Pll IC control for 65MHz clock
--				(C)Copyright 2014,03,15 by Y nishimura
-- **************************************************************************
LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity PLLIC65 is
	port
		(
			Hclk	: in  std_logic;	-- Master clock
			Reset	: in  std_logic;	-- Master Reset
			Cgate	: in  std_logic;	-- Slow clock
			Trig	: in  std_logic;	-- PLL set Trigger
			SCLK	: out std_logic;	-- PLL Serial clock
			DATA	: out std_logic;	-- PLL data
			BUSY	: out std_logic;	-- Data busy
			LE		: out std_logic		-- PLL Latch enable
		);
end PLLIC65;

architecture norm of PLLIC65 is
	signal	Sdata				: std_logic_vector(23 downto 0);
	signal	Stat				: std_logic_vector(7 downto 0);
	signal	Hstat				: std_logic_vector(1 downto 0);
	signal	Idle,Load,Lena,Ckk	: std_logic;
	signal	Rclk				: std_logic;

begin
	SCLK <= Rclk; DATA <= Sdata(23); LE <= Lena; BUSY <= Idle;
	process(Hclk,Reset)
	begin
		if Reset = '0' then Hstat <= "00";
		elsif rising_edge(Hclk) then
			if Cgate = '1'	then
				case Hstat is
					when "00" => If Trig = '1'	then Hstat <= "10";
												else Hstat <= "00";
								 end if;
					when "01" => If Idle = '0'	then Hstat <= "11";
												else Hstat <= "01";
								 end if;
					when "10" => Hstat <= "01";
					when "11" => Hstat <= "00";
					when others => Hstat <= "00";
				end case;
			else
				if Hstat = "00"	then
					if Trig = '1'				then Hstat <= "10";
												else Hstat <= Hstat;
					end if;
				else Hstat <= Hstat;
				end if;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--		Sequence counter
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then
			Stat <= "00000000"; Idle <= '0'; Ckk <= '0';
			Rclk <= '0';
		elsif rising_edge(Hclk) then
			if Cgate = '1'	then
				if Idle = '0'	then
					if Hstat(1) = '1'		then Idle <= '1';
											else Idle <= '0';
					end if;
				else
					if Stat = "11111111"	then Idle <= '0';
											else Idle <= '1';
					end if;
				end if;
				if Idle = '1'			then Stat <= Stat + 1;
										else Stat <= "00000000";
				end if;
				if Stat(5 downto 0) = "000001"	then Ckk <= '1';
				else
					if Stat(5 downto 0) = "110001"	then Ckk <= '0';
													else Ckk <= Ckk;
					end if;
				end if;
				if Ckk = '0'			then Rclk <= '0';
										else Rclk <= not Rclk;
				end if;
			else Stat <= Stat; Idle <= Idle; Ckk <= Ckk; Rclk <= Rclk;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--		Load timing
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then Load <= '0'; Lena <= '0';
		elsif rising_edge(Hclk) then
			if Cgate = '1'	then
				if Idle = '1'		then
					if Stat(5 downto 0) = "000001"	then Load <= '1';
													else Load <= '0';
					end if;
				else Load <= '0';
				end if;
				if Idle = '1'		then
					if Stat(5 downto 0) = "110010"	then Lena <= '1';
					else
						if Stat(0) = '1'			then Lena <= '0';
													else Lena <= Lena;
						end if;
					end if;
				else Lena <= '0';
				end if;
			else Load <= Load; Lena <= Lena;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--		Shift register
--				F=N*10MHz/R    N=130=0x82  R=20
--						F=65MHz
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' then Sdata <= "000000000000000000000000";
		elsif rising_edge(Hclk) then
			if Cgate = '1'	then
				if Load = '1'	then
					case Stat(7 downto 6) is
						when "00" => Sdata <= "001101000000000001010001";  -- R counter 1/20=500kHz
						when "01" => Sdata <= "000010010111000100100100";  -- Control reg
						when "10" => Sdata(23 downto 16) <= "00000000";
									 Sdata(15 downto 8) <= "10000010";
									 Sdata(7 downto 0) <= "00000010";   -- Set N counter
						when "11" => Sdata <= "001101000000000001010001";  -- R counter 1/20=500kHz
						when others => Sdata <= "000000000000000000000000";
					end case;
				else
					if Stat(0) = '1'	then
						 Sdata(23 downto 1) <= Sdata(22 downto 0);
						 Sdata(0) <= '0';
					else Sdata <= Sdata;
					end if;
				end if;
			else Sdata <= Sdata;
			end if;
		end if;
	end process;

end norm;

